@@ -51,11 +51,13 @@ import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.MojoFailureException;
 import org.apache.maven.plugin.logging.Log;
 import org.apache.maven.project.MavenProject;
+import org.apache.maven.project.MavenProjectHelper;
 import org.apache.maven.shared.osgi.Maven2OsgiConverter;
 import org.codehaus.plexus.archiver.UnArchiver;
 import org.codehaus.plexus.archiver.manager.ArchiverManager;
 import org.codehaus.plexus.util.DirectoryScanner;
 import org.codehaus.plexus.util.StringInputStream;
+import org.codehaus.plexus.util.StringUtils;
 
 import aQute.lib.osgi.Analyzer;
 import aQute.lib.osgi.Builder;
@@ -89,11 +91,24 @@ public class BundlePlugin extends AbstractMojo
     protected boolean unpackBundle;
 
     /**
-     * When true, exclude project dependencies from the classpath passed to BND
+     * Comma separated list of artifactIds to exclude from the dependency classpath passed to BND (use "true" to exclude everything)
      *
      * @parameter expression="${excludeDependencies}"
      */
-    protected boolean excludeDependencies;
+    protected String excludeDependencies;
+
+    /**
+     * Classifier type of the bundle to be installed.  For example, "jdk14".
+     * Defaults to none which means this is the project's main bundle.
+     *
+     * @parameter expression="${classifier}"
+     */
+    protected String classifier;
+
+    /**
+     * @component
+     */
+    private MavenProjectHelper m_projectHelper;
 
     /**
      * @component
@@ -147,7 +162,7 @@ public class BundlePlugin extends AbstractMojo
     private MavenProject project;
 
     /**
-     * The name of the generated JAR file.
+     * The BND instructions for the bundle.
      *
      * @parameter
      */
@@ -159,7 +174,7 @@ public class BundlePlugin extends AbstractMojo
     private Maven2OsgiConverter m_maven2OsgiConverter;
 
     private static final String MAVEN_RESOURCES = "{maven-resources}";
-    private static final String MAVEN_RESOURCES_REGEX = "\\{maven-resources\\}";
+
     private static final String[] EMPTY_STRING_ARRAY =
         {};
     private static final String[] DEFAULT_INCLUDES =
@@ -190,13 +205,13 @@ public class BundlePlugin extends AbstractMojo
     public void execute() throws MojoExecutionException
     {
         Properties properties = new Properties();
+        String projectType = getProject().getArtifact().getType();
 
-        // ignore project types not supported, useful when the plugin is configured in the parent pom
-        if ( !supportedProjectTypes.contains( getProject().getArtifact().getType() ) )
+        // ignore unsupported project types, useful when bundleplugin is configured in parent pom
+        if ( !supportedProjectTypes.contains( projectType ) )
         {
-            getLog().debug(
-                "Ignoring project " + getProject().getArtifact() + " : type " + getProject().getArtifact().getType()
-                    + " is not supported by bundle plugin, supported types are " + supportedProjectTypes );
+            getLog().warn(
+                "Ignoring project type " + projectType + " - supportedProjectTypes = " + supportedProjectTypes );
             return;
         }
 
@@ -285,8 +300,20 @@ public class BundlePlugin extends AbstractMojo
             // attach bundle to maven project
             jarFile.getParentFile().mkdirs();
             builder.getJar().write( jarFile );
-            Artifact bundleArtifact = currentProject.getArtifact();
-            bundleArtifact.setFile( jarFile );
+
+            Artifact mainArtifact = currentProject.getArtifact();
+
+            // workaround for MNG-1682: force maven to install artifact using the "jar" handler
+            mainArtifact.setArtifactHandler( m_artifactHandlerManager.getArtifactHandler( "jar" ) );
+
+            if ( null == classifier || classifier.trim().length() == 0 )
+            {
+                mainArtifact.setFile( jarFile );
+            }
+            else
+            {
+                m_projectHelper.attachArtifact( currentProject, jarFile, classifier );
+            }
 
             if ( unpackBundle )
             {
@@ -307,9 +334,6 @@ public class BundlePlugin extends AbstractMojo
                     getLog().error( "Error trying to write Manifest to file " + outputFile, e );
                 }
             }
-
-            // workaround for MNG-1682: force maven to install artifact using the "jar" handler
-            bundleArtifact.setArtifactHandler( m_artifactHandlerManager.getArtifactHandler( "jar" ) );
         }
         catch ( MojoFailureException e )
         {
@@ -330,20 +354,31 @@ public class BundlePlugin extends AbstractMojo
         properties.putAll( getDefaultProperties( currentProject ) );
         properties.putAll( transformDirectives( originalInstructions ) );
 
-        // update BND instructions to add Maven resources
-        includeMavenResources( currentProject, properties, getLog() );
-
         Builder builder = new Builder();
         builder.setBase( currentProject.getBasedir() );
         builder.setProperties( properties );
         builder.setClasspath( classpath );
 
+        // update BND instructions to add included Maven resources
+        includeMavenResources( currentProject, properties, getLog() );
+
         if ( !properties.containsKey( Analyzer.EXPORT_PACKAGE ) && !properties.containsKey( Analyzer.PRIVATE_PACKAGE ) )
         {
-            String bsn = properties.getProperty( Analyzer.BUNDLE_SYMBOLICNAME );
-            String namespace = bsn.replaceAll( "\\W", "." );
+            if ( properties.containsKey( Analyzer.EXPORT_CONTENTS ) )
+            {
+                /*
+                 * if we have exportcontents but no export packages or private packages then we're probably embedding or
+                 * inlining one or more jars, so set private package to a non-null (but empty) value to keep Bnd happy.
+                 */
+                properties.put( Analyzer.PRIVATE_PACKAGE, "!*" );
+            }
+            else
+            {
+                String bsn = properties.getProperty( Analyzer.BUNDLE_SYMBOLICNAME );
+                String namespace = bsn.replaceAll( "\\W", "." );
 
-            properties.put( Analyzer.EXPORT_PACKAGE, namespace + ".*" );
+                properties.put( Analyzer.EXPORT_PACKAGE, namespace + ".*" );
+            }
         }
 
         // update BND instructions to embed selected Maven dependencies
@@ -387,7 +422,8 @@ public class BundlePlugin extends AbstractMojo
                 }
                 else
                 {
-                    String combinedResource = includeResource.replaceAll( MAVEN_RESOURCES_REGEX, mavenResourcePaths );
+                    String combinedResource = StringUtils
+                        .replace( includeResource, MAVEN_RESOURCES, mavenResourcePaths );
                     properties.put( Analyzer.INCLUDE_RESOURCE, combinedResource );
                 }
             }
@@ -625,16 +661,7 @@ public class BundlePlugin extends AbstractMojo
             list.add( new Jar( ".", getOutputDirectory() ) );
         }
 
-        final Set artifacts;
-        if ( excludeDependencies )
-        {
-            artifacts = Collections.EMPTY_SET;
-        }
-        else
-        {
-            artifacts = currentProject.getArtifacts();
-        }
-
+        final Collection artifacts = getSelectedDependencies( currentProject.getArtifacts() );
         for ( Iterator it = artifacts.iterator(); it.hasNext(); )
         {
             Artifact artifact = ( Artifact ) it.next();
@@ -663,6 +690,33 @@ public class BundlePlugin extends AbstractMojo
     }
 
 
+    private Collection getSelectedDependencies( Set artifacts )
+    {
+        if ( null == excludeDependencies || excludeDependencies.length() == 0 )
+        {
+            return artifacts;
+        }
+        else if ( "true".equalsIgnoreCase( excludeDependencies ) )
+        {
+            return Collections.EMPTY_LIST;
+        }
+
+        List excludes = Arrays.asList( excludeDependencies.trim().split( "\\s*,\\s*" ) );
+
+        Collection classpath = new ArrayList();
+        for ( Iterator i = artifacts.iterator(); i.hasNext(); )
+        {
+            Artifact artifact = ( Artifact ) i.next();
+            if ( !excludes.contains( artifact.getArtifactId() ) )
+            {
+                classpath.add( artifact );
+            }
+        }
+
+        return classpath;
+    }
+
+
     /**
      * Get the file for an Artifact
      *
@@ -703,7 +757,13 @@ public class BundlePlugin extends AbstractMojo
      */
     protected String getBundleName( MavenProject currentProject )
     {
-        return currentProject.getBuild().getFinalName() + ".jar";
+        String finalName = currentProject.getBuild().getFinalName();
+        if ( null != classifier && classifier.trim().length() > 0 )
+        {
+            return finalName + '-' + classifier + ".jar";
+        }
+
+        return finalName + ".jar";
     }
 
 
@@ -767,6 +827,8 @@ public class BundlePlugin extends AbstractMojo
         properties.put( "project.build.directory", getBuildDirectory() );
         properties.put( "project.build.outputdirectory", getOutputDirectory() );
 
+        properties.put( "classifier", classifier == null ? "" : classifier );
+
         return properties;
     }
 
